@@ -18,15 +18,18 @@
 #include "structmember.h"
 #include "radix.h"
 
-/* $Id: radix_python.c,v 1.25 2004/11/24 20:46:18 djm Exp $ */
+/* $Id: radix_python.c,v 1.30 2007/12/18 02:49:01 djm Exp $ */
 
 /* Prototypes */
 struct _RadixObject;
 struct _RadixIterObject;
 static struct _RadixIterObject *newRadixIterObject(struct _RadixObject *);
+static PyObject *radix_Radix(PyObject *, PyObject *);
 
 /* ------------------------------------------------------------------------ */
 
+PyObject *radix_constructor;
+
 /* RadixNode: tree nodes */
 
 typedef struct {
@@ -68,7 +71,7 @@ newRadixNodeObject(radix_node_t *rn)
 	self->prefix = PyString_FromString(prefix);
 	self->prefixlen = PyInt_FromLong(rn->prefix->bitlen);
 	self->family = PyInt_FromLong(rn->prefix->family);
-	self->packed = PyString_FromStringAndSize((u_char*)&rn->prefix->add,
+	self->packed = PyString_FromStringAndSize((char*)&rn->prefix->add,
 	    rn->prefix->family == AF_INET ? 4 : 16);
 
 	if (self->user_attr == NULL || self->prefixlen == NULL || 
@@ -165,9 +168,10 @@ typedef struct _RadixObject {
 } RadixObject;
 
 static PyTypeObject Radix_Type;
+#define Radix_CheckExact(op) ((op)->ob_type == &Radix_Type)
 
 static RadixObject *
-newRadixObject(PyObject *arg)
+newRadixObject(void)
 {
 	RadixObject *self;
 	radix_tree_t *rt4, *rt6;
@@ -220,7 +224,8 @@ Radix_dealloc(RadixObject *self)
 static prefix_t
 *args_to_prefix(char *addr, char *packed, int packlen, long prefixlen)
 {
-	prefix_t *prefix;
+	prefix_t *prefix = NULL;
+	const char *errmsg;
 
 	if (addr != NULL && packed != NULL) {
 		PyErr_SetString(PyExc_TypeError,
@@ -235,12 +240,12 @@ static prefix_t
 	}
 
 	if (addr != NULL) {		/* Parse a string address */
-		if ((prefix = prefix_pton(addr, prefixlen)) == NULL) {
-			PyErr_SetString(PyExc_ValueError,
+		if ((prefix = prefix_pton(addr, prefixlen, &errmsg)) == NULL) {
+			PyErr_SetString(PyExc_ValueError, errmsg ? errmsg :
 			    "Invalid address format");
 		}
 	} else if (packed != NULL) {	/* "parse" a packed binary address */
-		if ((prefix = prefix_from_blob(packed, packlen, 
+		if ((prefix = prefix_from_blob((u_char*)packed, packlen, 
 		    prefixlen)) == NULL) {
 			PyErr_SetString(PyExc_ValueError,
 			    "Invalid packed address format");
@@ -255,6 +260,37 @@ static prefix_t
 	return prefix;
 }
 
+#define PICKRT(prefix, rno) (prefix->family == AF_INET6 ? rno->rt6 : rno->rt4)
+
+static PyObject *
+create_add_node(RadixObject *self, prefix_t *prefix)
+{
+	radix_node_t *node;
+	RadixNodeObject *node_obj;
+
+	if ((node = radix_lookup(PICKRT(prefix, self), prefix)) == NULL) {
+		PyErr_SetString(PyExc_MemoryError, "Couldn't add prefix");
+		return NULL;
+	}
+
+	/*
+	 * Create a RadixNode object in the data area of the node
+	 * We duplicate most of the node's identity, because the radix.c:node 
+	 * itself has a lifetime independent of the Python node object
+	 * Confusing? yeah...
+	 */
+	if (node->data == NULL) {
+		if ((node_obj = newRadixNodeObject(node)) == NULL)
+			return (NULL);
+		node->data = node_obj;
+	} else
+		node_obj = node->data;
+
+	self->gen_id++;
+	Py_XINCREF(node_obj);
+	return (PyObject *)node_obj;
+}
+
 PyDoc_STRVAR(Radix_add_doc,
 "Radix.add(network[, masklen][, packed]) -> new RadixNode object\n\
 \n\
@@ -274,15 +310,12 @@ the same tree.\n\
 This method returns a RadixNode object. Arbitrary data may be strored\n\
 in the RadixNode.data dict.");
 
-#define PICKRT(prefix, rno) (prefix->family == AF_INET6 ? rno->rt6 : rno->rt4)
-
 static PyObject *
 Radix_add(RadixObject *self, PyObject *args, PyObject *kw_args)
 {
 	prefix_t *prefix;
-	radix_node_t *node;
-	RadixNodeObject *node_obj;
 	static char *keywords[] = { "network", "masklen", "packed", NULL };
+	PyObject *node_obj;
 
 	char *addr = NULL, *packed = NULL;
 	long prefixlen = -1;
@@ -294,29 +327,10 @@ Radix_add(RadixObject *self, PyObject *args, PyObject *kw_args)
 	if ((prefix = args_to_prefix(addr, packed, packlen, prefixlen)) == NULL)
 		return NULL;
 
-	if ((node = radix_lookup(PICKRT(prefix, self), prefix)) == NULL) {
-		Deref_Prefix(prefix);
-		PyErr_SetString(PyExc_MemoryError, "Couldn't add prefix");
-		return NULL;
-	}
+	node_obj = create_add_node(self, prefix);
 	Deref_Prefix(prefix);
 
-	/*
-	 * Create a RadixNode object in the data area of the node
-	 * We duplicate most of the node's identity, because the radix.c:node 
-	 * itself has a lifetime indepenant of the Python node object
-	 * Confusing? yeah...
-	 */
-	if (node->data == NULL) {
-		if ((node_obj = newRadixNodeObject(node)) == NULL)
-			return (NULL);
-		node->data = node_obj;
-	} else
-		node_obj = node->data;
-
-	self->gen_id++;
-	Py_XINCREF(node_obj);
-	return (PyObject *)node_obj;
+	return node_obj;
 }
 
 PyDoc_STRVAR(Radix_delete_doc,
@@ -505,6 +519,114 @@ Radix_prefixes(RadixObject *self, PyObject *args)
 	return (ret);
 }
 
+/* Used for pickling */
+static PyObject *
+radix_getstate(RadixObject *self)
+{
+	radix_node_t *node;
+	PyObject *ret;
+	RadixNodeObject *rnode;
+
+	if ((ret = PyList_New(0)) == NULL)
+		return NULL;
+
+	RADIX_WALK(self->rt4->head, node) {
+		if (node->data != NULL) {
+			rnode = (RadixNodeObject *)node->data;
+			PyList_Append(ret, Py_BuildValue("(OO)",
+			    rnode->prefix, rnode->user_attr));
+			Py_INCREF(rnode->prefix);
+			Py_INCREF(rnode->user_attr);
+		}
+	} RADIX_WALK_END;
+	RADIX_WALK(self->rt6->head, node) {
+		if (node->data != NULL) {
+			rnode = (RadixNodeObject *)node->data;
+			PyList_Append(ret, Py_BuildValue("(OO)",
+			    rnode->prefix, rnode->user_attr));
+			Py_INCREF(rnode->prefix);
+			Py_INCREF(rnode->user_attr);
+		}
+	} RADIX_WALK_END;
+
+	return (ret);
+}
+
+static PyObject *
+Radix_getstate(RadixObject *self, PyObject *args)
+{
+	if (!PyArg_ParseTuple(args, ":__getstate__"))
+		return NULL;
+	return radix_getstate(self);
+}
+
+static PyObject *
+Radix_reduce(RadixObject *self, PyObject *args)
+{
+	PyObject *state, *ret;
+
+	if (!PyArg_ParseTuple(args, ":__reduce__"))
+		return NULL;
+	if ((state = radix_getstate(self)) == NULL)
+		return NULL;
+
+	ret = Py_BuildValue("(O()O)", radix_constructor, state);
+	Py_XINCREF(radix_constructor);
+	Py_XINCREF(state);
+
+	return ret;
+}
+
+/* Used for unpickling */
+static PyObject *
+Radix_setstate(RadixObject *self, PyObject *args)
+{
+	PyObject *state, *tpl, *addr, *data;
+	int len, i;
+	RadixNodeObject *node;
+	prefix_t *prefix;
+	char *addr_string;
+	const char *errmsg;
+
+	if (!Radix_CheckExact(self)) {
+		PyErr_SetString(PyExc_ValueError, "not a Radix object");
+		return NULL;
+	}
+
+	/* XXX type-checking is overstrict here */
+	if (!PyArg_ParseTuple(args, "O!:__setstate__", &PyList_Type, &state))
+		return NULL;
+
+	len = PyList_Size(state);
+	for (i = 0; i < len; i++) {
+		if ((tpl = PyList_GetItem(state, i)) == NULL)
+			return NULL;
+		if ((addr = PyTuple_GetItem(tpl, 0)) == NULL)
+			return NULL;
+		if ((data = PyTuple_GetItem(tpl, 1)) == NULL)
+			return NULL;
+		if ((addr_string = PyString_AsString(addr)) == NULL)
+			return NULL;
+		if ((prefix = prefix_pton(addr_string, -1, &errmsg)) == NULL) {
+			PyErr_SetString(PyExc_ValueError, errmsg ? errmsg :
+			    "Invalid address format");
+			return NULL;
+		}
+		if ((node = (RadixNodeObject *)create_add_node(self,
+		    prefix)) == NULL) {
+			Deref_Prefix(prefix);
+			return NULL;
+		}
+		Deref_Prefix(prefix);
+		Py_XDECREF(node->user_attr);
+		node->user_attr = data;
+		Py_INCREF(node->user_attr);
+	}
+
+	Py_INCREF(Py_None);
+	return Py_None;
+}
+
 static PyObject *
 Radix_getiter(RadixObject *self)
 {
@@ -520,6 +642,9 @@ static PyMethodDef Radix_methods[] = {
 	{"search_best",	(PyCFunction)Radix_search_best,	METH_VARARGS|METH_KEYWORDS,	Radix_search_best_doc	},
 	{"nodes",	(PyCFunction)Radix_nodes,	METH_VARARGS,			Radix_nodes_doc		},
 	{"prefixes",	(PyCFunction)Radix_prefixes,	METH_VARARGS,			Radix_prefixes_doc	},
+	{"__getstate__",(PyCFunction)Radix_getstate,	METH_VARARGS,			NULL			},
+	{"__setstate__",(PyCFunction)Radix_setstate,	METH_VARARGS,			NULL			},
+	{"__reduce__",	(PyCFunction)Radix_reduce,	METH_VARARGS,			NULL			},
 	{NULL,		NULL}		/* sentinel */
 };
 
@@ -724,7 +849,7 @@ radix_Radix(PyObject *self, PyObject *args)
 
 	if (!PyArg_ParseTuple(args, ":Radix"))
 		return NULL;
-	rv = newRadixObject(args);
+	rv = newRadixObject();
 	if (rv == NULL)
 		return NULL;
 	return (PyObject *)rv;
@@ -736,91 +861,128 @@ static PyMethodDef radix_methods[] = {
 };
 
 PyDoc_STRVAR(module_doc,
-"Implementation of a radix tree data structure for network prefixes.\n\
-\n\
-The radix tree is the data structure most commonly used for routing\n\
-table lookups. It efficiently stores network prefixes of varying\n\
-lengths and allows fast lookups of containing networks.\n\
-\n\
-Simple example:\n\
-\n\
-	import radix\n\
-\n\
-	# Create a new tree\n\
-	rtree = radix.Radix()\n\
-\n\
-	# Adding a node returns a RadixNode object. You can create\n\
-	# arbitrary members in its 'data' dict to store your data\n\
-	rnode = rtree.add(\"10.0.0.0/8\")\n\
-	rnode.data[\"blah\"] = \"whatever you want\"\n\
-\n\
-	# You can specify nodes as CIDR addresses, or networks with\n\
-	# separate mask lengths. The following three invocations are\n\
-	# identical:\n\
-	rnode = rtree.add(\"10.0.0.0/16\")\n\
-	rnode = rtree.add(\"10.0.0.0\", 16)\n\
-	rnode = rtree.add(network = \"10.0.0.0\", masklen = 16)\n\
-\n\
-	# It is also possible to specify nodes using binary packed\n\
-	# addresses, such as those returned by the socket module\n\
-	# functions. In this case, the radix module will assume that\n\
-	# a four-byte address is an IPv4 address and a sixteen-byte\n\
-	# address is an IPv6 address. For example:\n\
-	binary_addr = inet_ntoa(\"172.18.22.0\")\n\
-	rnode = rtree.add(packed = binary_addr, masklen = 23)\n\
-\n\
-	# Exact search will only return prefixes you have entered\n\
-	# You can use all of the above ways to specify the address\n\
-	rnode = rtree.search_exact(\"10.0.0.0/8\")\n\
-	# Get your data back out\n\
-	print rnode.data[\"blah\"]\n\
-	# Use a packed address\n\
-	addr = socket.inet_ntoa(\"10.0.0.0\")\n\
-	rnode = rtree.search_exact(packed = addr, masklen = 8)\n\
-\n\
-	# Best-match search will return the longest matching prefix\n\
-	# that contains the search term (routing-style lookup)\n\
-	rnode = rtree.search_best(\"10.123.45.6\")\n\
-\n\
-	# There are a couple of implicit members of a RadixNode:\n\
-	print rnode.network	# -> \"10.0.0.0\"\n\
-	print rnode.prefix	# -> \"10.0.0.0/8\"\n\
-	print rnode.prefixlen	# -> 8\n\
-	print rnode.family	# -> socket.AF_INET\n\
-	print rnode.packed	# -> '\\n\\x00\\x00\\x00'\n\
-\n\
-	# IPv6 prefixes are fully supported in the same tree\n\
-	rnode = rtree.add(\"2001:DB8::/32\")\n\
-	rnode = rtree.add(\"::/0\")\n\
-\n\
-	# Use the nodes() method to return all RadixNodes created\n\
-	nodes = rtree.nodes()\n\
-	for rnode in nodes:\n\
-  		print rnode.prefix\n\
-\n\
-	# The prefixes() method will return all the prefixes (as a\n\
-	# list of strings) that have been entered\n\
-	prefixes = rtree.prefixes()\n\
-\n\
-	# You can also directly iterate over the tree itself\n\
-	# this would save some memory if the tree is big\n\
-	# NB. Don't modify the tree (add or delete nodes) while\n\
-	# iterating otherwise you will abort the iteration and\n\
-	# receive a RuntimeWarning. Changing a node's data dict\n\
-	# is permitted.\n\
-	for rnode in rtree:\n\
-  		print rnode.prefix\n\
-");
+"Implementation of a radix tree data structure for network prefixes.\n"
+"\n"
+"The radix tree is the data structure most commonly used for routing\n"
+"table lookups. It efficiently stores network prefixes of varying\n"
+"lengths and allows fast lookups of containing networks.\n"
+"\n"
+"Simple example:\n"
+"\n"
+"	import radix\n"
+"\n"
+"	# Create a new tree\n"
+"	rtree = radix.Radix()\n"
+"\n"
+"	# Adding a node returns a RadixNode object. You can create\n"
+"	# arbitrary members in its 'data' dict to store your data\n"
+"	rnode = rtree.add(\"10.0.0.0/8\")\n"
+"	rnode.data[\"blah\"] = \"whatever you want\"\n"
+"\n"
+"	# You can specify nodes as CIDR addresses, or networks with\n"
+"	# separate mask lengths. The following three invocations are\n"
+"	# identical:\n"
+"	rnode = rtree.add(\"10.0.0.0/16\")\n"
+"	rnode = rtree.add(\"10.0.0.0\", 16)\n"
+"	rnode = rtree.add(network = \"10.0.0.0\", masklen = 16)\n"
+"\n"
+"	# It is also possible to specify nodes using binary packed\n"
+"	# addresses, such as those returned by the socket module\n"
+"	# functions. In this case, the radix module will assume that\n"
+"	# a four-byte address is an IPv4 address and a sixteen-byte\n"
+"	# address is an IPv6 address. For example:\n"
+"	binary_addr = inet_ntoa(\"172.18.22.0\")\n"
+"	rnode = rtree.add(packed = binary_addr, masklen = 23)\n"
+"\n"
+"	# Exact search will only return prefixes you have entered\n"
+"	# You can use all of the above ways to specify the address\n"
+"	rnode = rtree.search_exact(\"10.0.0.0/8\")\n"
+"	# Get your data back out\n"
+"	print rnode.data[\"blah\"]\n"
+"	# Use a packed address\n"
+"	addr = socket.inet_ntoa(\"10.0.0.0\")\n"
+"	rnode = rtree.search_exact(packed = addr, masklen = 8)\n"
+"\n"
+"	# Best-match search will return the longest matching prefix\n"
+"	# that contains the search term (routing-style lookup)\n"
+"	rnode = rtree.search_best(\"10.123.45.6\")\n"
+"\n"
+"	# There are a couple of implicit members of a RadixNode:\n"
+"	print rnode.network	# -> \"10.0.0.0\"\n"
+"	print rnode.prefix	# -> \"10.0.0.0/8\"\n"
+"	print rnode.prefixlen	# -> 8\n"
+"	print rnode.family	# -> socket.AF_INET\n"
+"	print rnode.packed	# -> '\\n\\x00\\x00\\x00'\n"
+"\n"
+"	# IPv6 prefixes are fully supported in the same tree\n"
+"	rnode = rtree.add(\"2001:DB8::/32\")\n"
+"	rnode = rtree.add(\"::/0\")\n"
+"\n"
+"	# Use the nodes() method to return all RadixNodes created\n"
+"	nodes = rtree.nodes()\n"
+"	for rnode in nodes:\n"
+"  		print rnode.prefix\n"
+"\n"
+"	# The prefixes() method will return all the prefixes (as a\n"
+"	# list of strings) that have been entered\n"
+"	prefixes = rtree.prefixes()\n"
+"\n"
+"	# You can also directly iterate over the tree itself\n"
+"	# this would save some memory if the tree is big\n"
+"	# NB. Don't modify the tree (add or delete nodes) while\n"
+"	# iterating otherwise you will abort the iteration and\n"
+"	# receive a RuntimeWarning. Changing a node's data dict\n"
+"	# is permitted.\n"
+"	for rnode in rtree:\n"
+"  		print rnode.prefix\n"
+);
+
+#if defined(_MSC_VER)
+static void cleanupradix(void)
+{
+	WSACleanup();
+}
+#endif
 
 PyMODINIT_FUNC
 initradix(void)
 {
-	PyObject *m;
+	PyObject *m, *d;
+#if defined(_MSC_VER)
+	WSADATA winsock_data;
+	int r;
+	char errbuf[256];
+
+	r = WSAStartup(0x0202, &winsock_data);
+	switch (r) {
+	case 0:
+		Py_AtExit(cleanupradix);
+		break;
+	case WSASYSNOTREADY:
+		PyErr_SetString(PyExc_ImportError, "Winsock error: "
+		    "network subsystem not ready");
+		return;
+	case WSAEINVAL:
+	case WSAVERNOTSUPPORTED:
+		PyErr_SetString(PyExc_ImportError, "Winsock error: "
+		    "required winsock version 2.2 not supported");
+		return;
+	default:
+		snprintf(errbuf, sizeof(errbuf), "Winsock error: code %d", r);
+		PyErr_SetString(PyExc_ImportError, errbuf);
+		return;
+	}
+#endif
 
 	if (PyType_Ready(&Radix_Type) < 0)
 		return;
 	if (PyType_Ready(&RadixNode_Type) < 0)
 		return;
 	m = Py_InitModule3("radix", radix_methods, module_doc);
-	PyModule_AddStringConstant(m, "__version__", PROGVER);
+
+	/* Stash the callable constructor for use in Radix.__reduce__ */
+	d = PyModule_GetDict(m);
+	radix_constructor = PyDict_GetItemString(d, "Radix");
+
+	PyModule_AddStringConstant(m, "__version__", "0.4");
 }
